#! /usr/bin/python2
# -*- coding: utf8 -*-

import unittest
import mock
import optionsresolver
import ConfigParser


class OptionsResolverTestCase(unittest.TestCase):
    def setUp(self):
        optionsresolver.os_real = optionsresolver.os
        optionsresolver.optparse.sys.argv[1:] = []

        # Stash the original function to reassign it later
        self.ConfigParser = optionsresolver.ConfigParser.ConfigParser

    def tearDown(self):
        optionsresolver.os = optionsresolver.os_real
        optionsresolver.optparse.sys.argv[1:] = []
        optionsresolver.ConfigParser.ConfigParser = self.ConfigParser

    def test_setup_options_returns_options_when_options_given_and_no_config_file(self):
        resolver = optionsresolver.OptionsResover()
        options = [
            "--host", "foo-host",
            "--port", "foo-port",
            "--username", "foo-username",
            "--password", "foo-password",
            "--vhost", "foo-vhost",
            "--queues", "foo-queue",
            "--check-rate", "10",
            "--ready-queue-size", "20",
            "--unacknowledged-queue-size", "30",
            "--total-queue-size", "40",
            "--consumers-connected", "50",
            "--open-connections", "51",
            "--nodes-running", "60",
            "--node-memory-used", "70",
            "--email-to", "foo-email-to",
            "--email-from", "foo-email-from",
            "--email-subject", "foo-email-subject",
            "--email-server", "foo-email-server",
            "--email-password", "foo-email-password",
            "--email-ssl", "",
            "--slack-url", "foo-slack-url",
            "--slack-channel", "foo-slack-channel",
            "--slack-username", "foo-slack-username",
            "--telegram-bot-id", "foo-telegram-bot-id",
            "--telegram-channel", "foo-telegram-channel"
        ]

        optionsresolver.optparse.sys.argv[1:] = options
        options_result = resolver.setup_options()

        self.assertEquals("foo-host", options_result["host"])
        self.assertEquals("foo-port", options_result["port"])
        self.assertEquals("foo-username", options_result["username"])
        self.assertEquals("foo-password", options_result["password"])
        self.assertEquals("foo-vhost", options_result["vhost"])
        self.assertEquals(["foo-queue"], options_result["queues"])
        self.assertEquals(10, options_result["check_rate"])
        self.assertEquals(20, options_result["conditions"]["foo-queue"]["ready_queue_size"])
        self.assertEquals(30, options_result["conditions"]["foo-queue"]["unack_queue_size"])
        self.assertEquals(40, options_result["conditions"]["foo-queue"]["total_queue_size"])
        self.assertEquals(50, options_result["conditions"]["foo-queue"]["consumers_connected"])
        self.assertEquals(51, options_result["conditions"]["foo-queue"]["open_connections"])
        self.assertEquals(60, options_result["conditions"]["foo-queue"]["nodes_running"])
        self.assertEquals(70, options_result["conditions"]["foo-queue"]["node_memory_used"])
        self.assertEquals(["foo-email-to"], options_result["email_to"])
        self.assertEquals("foo-email-from", options_result["email_from"])
        self.assertEquals("foo-email-subject", options_result["email_subject"])
        self.assertEquals("foo-email-server", options_result["email_server"])
        self.assertEquals("foo-email-password", options_result["email_password"])
        self.assertEquals(True, options_result["email_ssl"])
        self.assertEquals("foo-slack-url", options_result["slack_url"])
        self.assertEquals("foo-slack-channel", options_result["slack_channel"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-telegram-bot-id", options_result["telegram_bot_id"])
        self.assertEquals("foo-telegram-channel", options_result["telegram_channel"])

    def test_setup_options_without_ssl_return_options_when_options_without_ssl_given_and_no_config_file(self):
        resolver = optionsresolver.OptionsResover()
        options = [
            "--host", "foo-host",
            "--port", "foo-port",
            "--username", "foo-username",
            "--password", "foo-password",
            "--vhost", "foo-vhost",
            "--queues", "foo-queue",
            "--check-rate", "10",
            "--ready-queue-size", "20",
            "--unacknowledged-queue-size", "30",
            "--total-queue-size", "40",
            "--consumers-connected", "50",
            "--open-connections", "51",
            "--nodes-running", "60",
            "--node-memory-used", "70",
            "--email-to", "foo-email-to",
            "--email-from", "foo-email-from",
            "--email-subject", "foo-email-subject",
            "--email-server", "foo-email-server",
            "--email-password", "foo-email-password",
            "--slack-url", "foo-slack-url",
            "--slack-channel", "foo-slack-channel",
            "--slack-username", "foo-slack-username",
            "--telegram-bot-id", "foo-telegram-bot-id",
            "--telegram-channel", "foo-telegram-channel"
        ]

        optionsresolver.optparse.sys.argv[1:] = options
        options_result = resolver.setup_options()

        self.assertEquals("foo-host", options_result["host"])
        self.assertEquals("foo-port", options_result["port"])
        self.assertEquals("foo-username", options_result["username"])
        self.assertEquals("foo-password", options_result["password"])
        self.assertEquals("foo-vhost", options_result["vhost"])
        self.assertEquals(["foo-queue"], options_result["queues"])
        self.assertEquals(10, options_result["check_rate"])
        self.assertEquals(20, options_result["conditions"]["foo-queue"]["ready_queue_size"])
        self.assertEquals(30, options_result["conditions"]["foo-queue"]["unack_queue_size"])
        self.assertEquals(40, options_result["conditions"]["foo-queue"]["total_queue_size"])
        self.assertEquals(50, options_result["conditions"]["foo-queue"]["consumers_connected"])
        self.assertEquals(51, options_result["conditions"]["foo-queue"]["open_connections"])
        self.assertEquals(60, options_result["conditions"]["foo-queue"]["nodes_running"])
        self.assertEquals(70, options_result["conditions"]["foo-queue"]["node_memory_used"])
        self.assertEquals(["foo-email-to"], options_result["email_to"])
        self.assertEquals("foo-email-from", options_result["email_from"])
        self.assertEquals("foo-email-subject", options_result["email_subject"])
        self.assertEquals("foo-email-server", options_result["email_server"])
        self.assertEquals("foo-email-password", options_result["email_password"])
        self.assertEquals(False, options_result["email_ssl"])
        self.assertEquals("foo-slack-url", options_result["slack_url"])
        self.assertEquals("foo-slack-channel", options_result["slack_channel"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-telegram-bot-id", options_result["telegram_bot_id"])
        self.assertEquals("foo-telegram-channel", options_result["telegram_channel"])

    def test_setup_options_exits_with_error_when_config_file_not_found(self):
        resolver = optionsresolver.OptionsResover()
        options = ["--config", "foo.ini"]

        optionsresolver.os.path.isfile = mock.MagicMock(return_value=False)
        optionsresolver.optparse.sys.argv[1:] = options

        with self.assertRaises(SystemExit) as se:
            resolver.setup_options()

        self.assertEqual(se.exception.code, 1)

    def test_setup_options_returns_options_when_config_file_found(self):
        options = ["--config", "foo.ini"]

        config_file_options = self.construct_config_file_options_with_generic_conditions()
        parser = ConfigParser.ConfigParser()
        parser._sections = config_file_options

        optionsresolver.os.path.isfile = mock.MagicMock(return_value=True)
        optionsresolver.ConfigParser.ConfigParser = mock.MagicMock(return_value=parser)
        optionsresolver.optparse.sys.argv[1:] = options
        options_result = optionsresolver.OptionsResover.setup_options()

        self.assertEquals("foo-host", options_result["host"])
        self.assertEquals("foo-port", options_result["port"])
        self.assertEquals("foo-username", options_result["username"])
        self.assertEquals("foo-password", options_result["password"])
        self.assertEquals("foo-vhost", options_result["vhost"])
        self.assertEquals(["foo-queue"], options_result["queues"])
        self.assertEquals(10, options_result["check_rate"])
        self.assertEquals(20, options_result["conditions"]["foo-queue"]["ready_queue_size"])
        self.assertEquals(30, options_result["conditions"]["foo-queue"]["unack_queue_size"])
        self.assertEquals(40, options_result["conditions"]["foo-queue"]["total_queue_size"])
        self.assertEquals(50, options_result["conditions"]["foo-queue"]["consumers_connected"])
        self.assertEquals(51, options_result["conditions"]["foo-queue"]["open_connections"])
        self.assertEquals(60, options_result["conditions"]["foo-queue"]["nodes_running"])
        self.assertEquals(70, options_result["conditions"]["foo-queue"]["node_memory_used"])
        self.assertEquals(["foo-email-to"], options_result["email_to"])
        self.assertEquals("foo-email-from", options_result["email_from"])
        self.assertEquals("foo-email-subject", options_result["email_subject"])
        self.assertEquals("foo-email-host", options_result["email_server"])
        self.assertEquals("foo-email-password", options_result["email_password"])
        self.assertEquals(True, options_result["email_ssl"])
        self.assertEquals("foo-slack-url", options_result["slack_url"])
        self.assertEquals("foo-slack-channel", options_result["slack_channel"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-telegram-bot-id", options_result["telegram_bot_id"])
        self.assertEquals("foo-telegram-channel", options_result["telegram_channel"])

    def test_setup_options_overrides_config_file_option_when_option_given_from_cli(self):
        options = ["--config", "foo.ini", "--email-to", "foo-email-to-new"]

        config_file_options = self.construct_config_file_options_with_generic_conditions()
        parser = ConfigParser.ConfigParser()
        parser._sections = config_file_options

        optionsresolver.os.path.isfile = mock.MagicMock(return_value=True)
        optionsresolver.ConfigParser.ConfigParser = mock.MagicMock(return_value=parser)
        optionsresolver.optparse.sys.argv[1:] = options
        options_result = optionsresolver.OptionsResover.setup_options()

        self.assertEquals(["foo-email-to-new"], options_result["email_to"])

    def test_setup_options_returns_options_when_config_file_found_with_queue_specific_conditions(self):
        options = ["--config", "foo.ini"]

        config_file_options = self.construct_config_file_options_with_queue_specific_conditions()
        parser = ConfigParser.ConfigParser()
        parser._sections = config_file_options

        optionsresolver.os.path.isfile = mock.MagicMock(return_value=True)
        optionsresolver.ConfigParser.ConfigParser = mock.MagicMock(return_value=parser)
        optionsresolver.optparse.sys.argv[1:] = options
        options_result = optionsresolver.OptionsResover.setup_options()

        self.assertEquals("foo-host", options_result["host"])
        self.assertEquals("foo-port", options_result["port"])
        self.assertEquals("foo-username", options_result["username"])
        self.assertEquals("foo-password", options_result["password"])
        self.assertEquals("foo-vhost", options_result["vhost"])
        self.assertEquals(["foo-queue"], options_result["queues"])
        self.assertEquals(10, options_result["check_rate"])
        self.assertEquals(20, options_result["conditions"]["foo-queue"]["ready_queue_size"])
        self.assertEquals(30, options_result["conditions"]["foo-queue"]["unack_queue_size"])
        self.assertEquals(40, options_result["conditions"]["foo-queue"]["total_queue_size"])
        self.assertEquals(50, options_result["conditions"]["foo-queue"]["consumers_connected"])
        self.assertEquals(51, options_result["conditions"]["foo-queue"]["open_connections"])
        self.assertEquals(60, options_result["conditions"]["foo-queue"]["nodes_running"])
        self.assertEquals(70, options_result["conditions"]["foo-queue"]["node_memory_used"])
        self.assertEquals(["foo-email-to"], options_result["email_to"])
        self.assertEquals("foo-email-from", options_result["email_from"])
        self.assertEquals("foo-email-subject", options_result["email_subject"])
        self.assertEquals("foo-email-host", options_result["email_server"])
        self.assertEquals("foo-email-password", options_result["email_password"])
        self.assertEquals(True, options_result["email_ssl"])
        self.assertEquals("foo-slack-url", options_result["slack_url"])
        self.assertEquals("foo-slack-channel", options_result["slack_channel"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-telegram-bot-id", options_result["telegram_bot_id"])
        self.assertEquals("foo-telegram-channel", options_result["telegram_channel"])

    def test_setup_options_returns_options_when_config_file_found_with_multiple_queue_specific_conditions(self):
        options = ["--config", "foo.ini"]

        config_file_options = self.construct_config_file_options_with_multiple_queue_specific_conditions()
        parser = ConfigParser.ConfigParser()
        parser._sections = config_file_options

        optionsresolver.os.path.isfile = mock.MagicMock(return_value=True)
        optionsresolver.ConfigParser.ConfigParser = mock.MagicMock(return_value=parser)
        optionsresolver.optparse.sys.argv[1:] = options
        options_result = optionsresolver.OptionsResover.setup_options()

        self.assertEquals("foo-host", options_result["host"])
        self.assertEquals("foo-port", options_result["port"])
        self.assertEquals("foo-username", options_result["username"])
        self.assertEquals("foo-password", options_result["password"])
        self.assertEquals("foo-vhost", options_result["vhost"])
        self.assertEquals(["foo-queue", "bar-queue"], options_result["queues"])
        self.assertEquals(10, options_result["check_rate"])
        self.assertEquals(20, options_result["conditions"]["foo-queue"]["ready_queue_size"])
        self.assertEquals(30, options_result["conditions"]["foo-queue"]["unack_queue_size"])
        self.assertEquals(40, options_result["conditions"]["foo-queue"]["total_queue_size"])
        self.assertEquals(50, options_result["conditions"]["foo-queue"]["consumers_connected"])
        self.assertEquals(51, options_result["conditions"]["foo-queue"]["open_connections"])
        self.assertEquals(60, options_result["conditions"]["foo-queue"]["nodes_running"])
        self.assertEquals(70, options_result["conditions"]["foo-queue"]["node_memory_used"])
        self.assertEquals(30, options_result["conditions"]["bar-queue"]["ready_queue_size"])
        self.assertEquals(40, options_result["conditions"]["bar-queue"]["unack_queue_size"])
        self.assertEquals(50, options_result["conditions"]["bar-queue"]["total_queue_size"])
        self.assertEquals(60, options_result["conditions"]["bar-queue"]["consumers_connected"])
        self.assertEquals(61, options_result["conditions"]["bar-queue"]["open_connections"])
        self.assertEquals(70, options_result["conditions"]["bar-queue"]["nodes_running"])
        self.assertEquals(80, options_result["conditions"]["bar-queue"]["node_memory_used"])
        self.assertEquals(["foo-email-to"], options_result["email_to"])
        self.assertEquals("foo-email-from", options_result["email_from"])
        self.assertEquals("foo-email-subject", options_result["email_subject"])
        self.assertEquals("foo-email-host", options_result["email_server"])
        self.assertEquals("foo-email-password", options_result["email_password"])
        self.assertEquals(True, options_result["email_ssl"])
        self.assertEquals("foo-slack-url", options_result["slack_url"])
        self.assertEquals("foo-slack-channel", options_result["slack_channel"])
        self.assertEquals("foo-slack-username", options_result["slack_username"])
        self.assertEquals("foo-telegram-bot-id", options_result["telegram_bot_id"])
        self.assertEquals("foo-telegram-channel", options_result["telegram_channel"])

    @staticmethod
    def construct_config_file_options_with_generic_conditions():
        return {
            "Server": {
                "host": "foo-host",
                "port": "foo-port",
                "username": "foo-username",
                "password": "foo-password",
                "vhost": "foo-vhost",
                "queues": "foo-queue",
                "check_rate": "10"
            },
            "Conditions": {
                "ready_queue_size": "20",
                "unack_queue_size": "30",
                "total_queue_size": "40",
                "consumers_connected": "50",
                "open_connections": "51",
                "nodes_running": "60",
                "node_memory_used": "70"
            },
            "Email": {
                "to": "foo-email-to",
                "from": "foo-email-from",
                "subject": "foo-email-subject",
                "host": "foo-email-host",
                "password": "foo-email-password",
                "ssl": "True"
            },
            "Slack": {
                "url": "foo-slack-url",
                "channel": "foo-slack-channel",
                "username": "foo-slack-username"
            },
            "Telegram": {
                "bot_id": "foo-telegram-bot-id",
                "channel": "foo-telegram-channel"
            }
        }

    @staticmethod
    def construct_config_file_options_with_queue_specific_conditions():
        return {
            "Server": {
                "host": "foo-host",
                "port": "foo-port",
                "username": "foo-username",
                "password": "foo-password",
                "vhost": "foo-vhost",
                "queues": "foo-queue",
                "check_rate": "10"
            },
            "Conditions:foo-queue": {
                "ready_queue_size": "20",
                "unack_queue_size": "30",
                "total_queue_size": "40",
                "consumers_connected": "50",
                "open_connections": "51",
                "nodes_running": "60",
                "node_memory_used": "70"
            },
            "Email": {
                "to": "foo-email-to",
                "from": "foo-email-from",
                "subject": "foo-email-subject",
                "host": "foo-email-host",
                "password": "foo-email-password",
                "ssl": "True"
            },
            "Slack": {
                "url": "foo-slack-url",
                "channel": "foo-slack-channel",
                "username": "foo-slack-username"
            },
            "Telegram": {
                "bot_id": "foo-telegram-bot-id",
                "channel": "foo-telegram-channel"
            }
        }

    @staticmethod
    def construct_config_file_options_with_multiple_queue_specific_conditions():
        return {
            "Server": {
                "host": "foo-host",
                "port": "foo-port",
                "username": "foo-username",
                "password": "foo-password",
                "vhost": "foo-vhost",
                "queues": "foo-queue,bar-queue",
                "check_rate": "10"
            },
            "Conditions:foo-queue": {
                "ready_queue_size": "20",
                "unack_queue_size": "30",
                "total_queue_size": "40",
                "consumers_connected": "50",
                "open_connections": "51",
                "nodes_running": "60",
                "node_memory_used": "70"
            },
            "Conditions:bar-queue": {
                "ready_queue_size": "30",
                "unack_queue_size": "40",
                "total_queue_size": "50",
                "consumers_connected": "60",
                "open_connections": "61",
                "nodes_running": "70",
                "node_memory_used": "80"
            },
            "Email": {
                "to": "foo-email-to",
                "from": "foo-email-from",
                "subject": "foo-email-subject",
                "host": "foo-email-host",
                "password": "foo-email-password",
                "ssl": "True"
            },
            "Slack": {
                "url": "foo-slack-url",
                "channel": "foo-slack-channel",
                "username": "foo-slack-username"
            },
            "Telegram": {
                "bot_id": "foo-telegram-bot-id",
                "channel": "foo-telegram-channel"
            }
        }

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python2
# -*- coding: utf-8 -*-

import urllib2
import json
import time
import smtplib
import optionsresolver


class RabbitMQAlert:
    def check_queue_conditions(self, options):
        queue = options["queue"]
        url = "http://%s:%s/api/queues/%s/%s" % (options["host"], options["port"], options["vhost"], options["queue"])
        data = self.send_request(url, options)

        messages_ready = data.get("messages_ready")
        messages_unacknowledged = data.get("messages_unacknowledged")
        messages = data.get("messages")

        queue_conditions = options["conditions"][queue]
        ready_size = queue_conditions.get("ready_queue_size")
        unack_size = queue_conditions.get("unack_queue_size")
        total_size = queue_conditions.get("total_queue_size")

        if ready_size is not None and messages_ready > ready_size:
            self.send_notification(options, "%s: messages_ready > %s" % (queue, str(ready_size)))

        if unack_size is not None and messages_unacknowledged > unack_size:
            self.send_notification(options, "%s: messages_unacknowledged > %s" % (queue, str(unack_size)))

        if total_size is not None and messages > total_size:
            self.send_notification(options, "%s: messages > %s" % (queue, str(total_size)))

    def check_connection_conditions(self, options):
        queue = options["queue"]
        url = "http://%s:%s/api/connections" % (options["host"], options["port"])
        data = self.send_request(url, options)

        open_connections = len(data)

        open_connections_min = options["conditions"][queue].get("open_connections")

        if open_connections is not None and open_connections < open_connections_min:
            self.send_notification(options, "open_connections < %s" % str(open_connections_min))

    def check_consumer_conditions(self, options):
        queue = options["queue"]
        url = "http://%s:%s/api/consumers" % (options["host"], options["port"])
        data = self.send_request(url, options)

        consumers_connected = len(data)

        consumers_connected_min = options["conditions"][queue].get("consumers_connected")

        if consumers_connected is not None and consumers_connected < consumers_connected_min:
            self.send_notification(options, "consumers_connected < %s" % str(consumers_connected_min))

    def check_node_conditions(self, options):
        queue = options["queue"]
        url = "http://%s:%s/api/nodes" % (options["host"], options["port"])
        data = self.send_request(url, options)

        nodes_running = len(data)

        queue_conditions = options["conditions"][queue]
        nodes_run = queue_conditions.get("nodes_running")
        node_memory = queue_conditions.get("node_memory_used")

        if nodes_run is not None and nodes_running < nodes_run:
            self.send_notification(options, "nodes_running < %s" % str(nodes_run))

        for node in data:
            if node_memory is not None and node.get("mem_used") > (node_memory * 1000000):
                self.send_notification(options, "Node %s - node_memory_used > %s MBs" % (node.get("name"), str(node_memory)))

    def send_request(self, url, options):
        password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        password_mgr.add_password(None, url, options["username"], options["password"])
        handler = urllib2.HTTPBasicAuthHandler(password_mgr)
        opener = urllib2.build_opener(handler)

        try:
            request = opener.open(url)
            response = request.read()
            request.close()

            data = json.loads(response)
            return data

        except urllib2.HTTPError, e:
            print "Error code %s hitting %s" % (e.code, url)
            exit(1)

    def send_notification(self, options, body):
        text = "%s - %s" % (options["host"], body)

        if "email_to" in options and options["email_to"]:
            server = smtplib.SMTP(options["email_server"], 25)

            if "email_ssl" in options and options["email_ssl"]:
                server = smtplib.SMTP_SSL(options["email_server"], 465)

            if "email_password" in options and options["email_password"]:
                server.login(options["email_from"], options["email_password"])

            recipients = options["email_to"]
            # add subject as header before message text
            subject_email = options["email_subject"] % (options["host"], options["queue"])
            text_email = "Subject: %s\n\n%s" % (subject_email, text)
            server.sendmail(options["email_from"], recipients, text_email)

            server.quit()

        if "slack_url" in options and options["slack_url"] and "slack_channel" in options and options["slack_channel"] and "slack_username" in options and options["slack_username"]:
            # escape double quotes from possibly breaking the slack message payload
            text_slack = text.replace("\"", "\\\"")
            slack_payload = '{"channel": "#%s", "username": "%s", "text": "%s"}' % (options["slack_channel"], options["slack_username"], text_slack)

            request = urllib2.Request(options["slack_url"], slack_payload)
            response = urllib2.urlopen(request)
            response.close()

        if "telegram_bot_id" in options and options["telegram_bot_id"] and "telegram_channel" in options and options["telegram_channel"]:
            text_telegram = "%s: %s" % (options["queue"], text)
            telegram_url = "https://api.telegram.org/bot%s/sendMessage?chat_id=%s&text=%s" % (options["telegram_bot_id"], options["telegram_channel"], text_telegram)

            request = urllib2.Request(telegram_url)
            response = urllib2.urlopen(request)
            response.close()

if __name__ ==  "__main__":
    rabbitmqalert = RabbitMQAlert()
    options = optionsresolver.OptionsResover.setup_options()

    while True:
        for queue in options["queues"]:
            options["queue"] = queue
            queue_conditions = options["conditions"][queue]

            if "ready_queue_size" in queue_conditions\
                    or "unack_queue_size" in queue_conditions\
                    or "total_queue_size" in queue_conditions:
                rabbitmqalert.check_queue_conditions(options)

            if "open_connections" in queue_conditions:
                rabbitmqalert.check_connection_conditions(options)

            if "consumers_connected" in queue_conditions:
                rabbitmqalert.check_consumer_conditions(options)

            if "nodes_running" in queue_conditions:
                rabbitmqalert.check_node_conditions(options)

        time.sleep(options["check_rate"])



#! /usr/bin/python2
# -*- coding: utf8 -*-

import unittest
import mock
import rabbitmqalert
import optionsresolver


class RabbitMQAlertTestCase(unittest.TestCase):
    def setUp(self):
        rabbitmqalert.urllib2_real = rabbitmqalert.urllib2
        optionsresolver.OptionsResover.setup_options_real = optionsresolver.OptionsResover.setup_options

    def tearDown(self):
        rabbitmqalert.urllib2 = rabbitmqalert.urllib2_real
        optionsresolver.OptionsResover.setup_options = optionsresolver.OptionsResover.setup_options_real

    def test_check_queue_conditions_not_send_notification_when_not_exceeding_options(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_queue()
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_queue_conditions(options)

        rmqa.send_notification.assert_not_called()

    def test_check_queue_conditions_messages_ready_send_notification_when_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_queue()
        response["messages_ready"] = 2
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_queue_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_queue_conditions_messages_unacknowledged_send_notification_when_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_queue()
        response["messages_unacknowledged"] = 2
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_queue_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_queue_conditions_messages_send_notification_when_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_queue()
        response["messages"] = 2
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_queue_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_connection_conditions_open_connections_not_send_notification_when_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_connection()
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_connection_conditions(options)

        rmqa.send_notification.assert_not_called()

    def test_check_connection_conditions_open_connections_send_notification_when_beneath_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_connection()
        response.pop("connection_foo")
        response.pop("connection_bar")
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_connection_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_consumer_conditions_consumers_connected_not_send_notification_when_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_consumer()
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_consumer_conditions(options)

        rmqa.send_notification.assert_not_called()

    def test_check_consumer_conditions_consumers_connected_send_notification_when_beneath_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_consumer()
        response.pop("consumer_foo")
        response.pop("consumer_bar")
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_consumer_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_node_conditions_not_send_notification_when_normal(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_node()
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_node_conditions(options)

        rmqa.send_notification.assert_not_called()

    def test_check_node_conditions_send_notification_when_nodes_running_beneath_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_node()
        response.pop()
        response.pop()
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_node_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_check_node_conditions_send_notification_when_node_memory_exceeding_option(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        response = self.construct_response_node()
        response[0]["mem_used"] = 2000000
        rmqa.send_request = mock.MagicMock(return_value=response)

        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rmqa.send_notification = mock.MagicMock()
        rmqa.check_node_conditions(options)

        rmqa.send_notification.assert_called_once()

    def test_send_notification_sends_email_when_email_to_is_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.smtplib.SMTP().sendmail.assert_called_once()

    def test_send_notification_calles_login_when_email_password_is_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        options["email_password"] = "password"
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.smtplib.SMTP().login.assert_called_once()

    def test_send_notification_not_call_login_when_email_password_not_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.smtplib.SMTP().login.assert_not_called()

    def test_send_notification_sends_email_with_ssl_when_email_ssl_is_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        options["email_ssl"] = True
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.smtplib.SMTP_SSL().sendmail.assert_called_once()

    def test_send_notification_not_send_email_when_email_to_not_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        options["email_to"] = None
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.smtplib.SMTP().sendmail.assert_not_called()

    def test_send_notification_sends_to_slack_and_telegram_when_options_are_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        rabbitmqalert.urllib2.urlopen.assert_called()

    def test_send_notification_not_send_to_slack_when_any_option_not_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        options["slack_url"] = None
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        # Assure only telegram is called
        rabbitmqalert.urllib2.urlopen.assert_called_once()

    def test_send_notification_not_send_to_telegram_when_any_option_not_set(self):
        rmqa = rabbitmqalert.RabbitMQAlert()
        options = self.construct_options()
        options["telegram_bot_id"] = None
        optionsresolver.OptionsResover.setup_options = mock.MagicMock(return_value=options)

        rabbitmqalert.smtplib = mock.MagicMock()
        rabbitmqalert.urllib2 = mock.MagicMock()
        rmqa.send_notification(options, "")

        # Assure only slack is called
        rabbitmqalert.urllib2.urlopen.assert_called_once()

    @staticmethod
    def construct_options():
        options = {
            "host": "foo",
            "port": 1,
            "vhost": "foo",
            "queue": "foo",
            "queues": ["foo"],
            "conditions": {
                "foo": {
                    "ready_queue_size": 0,
                    "unack_queue_size": 0,
                    "total_queue_size": 0,
                    "consumers_connected": 1,
                    "open_connections": 1,
                    "nodes_running": 1,
                    "node_memory_used": 1
                }
            },
            "email_to": ["foo@foobar.com"],
            "email_from": "bar@foobar.com",
            "email_subject": "foo %s %s",
            "email_server": "mail.foobar.com",
            "email_password": "",
            "email_ssl": False,
            "slack_url": "http://foo.com",
            "slack_channel": "channel",
            "slack_username": "username",
            "telegram_bot_id": "foo_bot",
            "telegram_channel": "foo_channel"
        }

        return options

    @staticmethod
    def construct_response_queue():
        return {
            "messages_ready": 0,
            "messages_unacknowledged": 0,
            "messages": 0
        }

    @staticmethod
    def construct_response_connection():
        return {
            "connection_foo": {},
            "connection_bar": {}
        }

    @staticmethod
    def construct_response_consumer():
        return {
            "consumer_foo": {},
            "consumer_bar": {}
        }

    @staticmethod
    def construct_response_node():
        return [
            { "mem_used": 500000 },
            { "mem_used": 500000 }
        ]

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python2
# -*- coding: utf-8 -*-

import optparse
import ConfigParser
import os.path


class OptionsResover:
    @staticmethod
    def setup_options():
        arguments = optparse.OptionParser()
        arguments.add_option("-c", "--config-file", dest="config_file", help="Path of the configuration file", type="string")
        arguments.add_option("--host", dest="host", help="RabbitMQ API address", type="string")
        arguments.add_option("--port", dest="port", help="RabbitMQ API port", type="string")
        arguments.add_option("--username", dest="username", help="RabbitMQ API username", type="string")
        arguments.add_option("--password", dest="password", help="RabbitMQ API password", type="string")
        arguments.add_option("--vhost", dest="vhost", help="Name of the vhost to inspect", type="string")
        arguments.add_option("--queues", dest="queues", help="List of comma-separated queue names to inspect", type="string")
        arguments.add_option("--check-rate", dest="check_rate", help="Conditions check frequency, in seconds.", type="int")

        arguments.add_option("--ready-queue-size", dest="ready_queue_size", help="Size of Ready messages on the queue to alert as warning", type="int")
        arguments.add_option("--unacknowledged-queue-size", dest="unack_queue_size", help="Size of the Unacknowledged messages on the queue to alert as warning", type="int")
        arguments.add_option("--total-queue-size", dest="total_queue_size", help="Size of the Total messages on the queue to alert as warning", type="int")
        arguments.add_option("--consumers-connected", dest="consumers_connected", help="The number of consumers that should be connected", type="int")
        arguments.add_option("--open-connections", dest="open_connections", help="The number of open connections", type="int")
        arguments.add_option("--nodes-running", dest="nodes_running", help="The number of nodes running", type="int")
        arguments.add_option("--node-memory-used", dest="node_memory_used", help="Memory used by each node in MBs", type="int")

        arguments.add_option("--email-to", dest="email_to", help="List of comma-separated email addresses to send notification to", type="string")
        arguments.add_option("--email-from", dest="email_from", help="The sender email address", type="string")
        arguments.add_option("--email-subject", dest="email_subject", help="The email subject", type="string")
        arguments.add_option("--email-server", dest="email_server", help="The hostname or IP address of the mail server", type="string")
        arguments.add_option("--email-password", dest="email_password", help="The password for the authentication on the mail server", type="string")
        arguments.add_option("--email-ssl", dest="email_ssl", help="Use SSL to send email", action="store_true", default=False)
        arguments.add_option("--slack-url", dest="slack_url", help="Slack hook URL", type="string")
        arguments.add_option("--slack-channel", dest="slack_channel", help="Slack channel to message to", type="string")
        arguments.add_option("--slack-username", dest="slack_username", help="Sender's Slack username", type="string")
        arguments.add_option("--telegram-bot-id", dest="telegram_bot_id", help="Telegram bot id", type="string")
        arguments.add_option("--telegram-channel", dest="telegram_channel", help="Telegram channel", type="string")

        cli_arguments = arguments.parse_args()[0]

        # set as defaults the cli argument values
        config_file_options = ConfigParser.ConfigParser(vars(cli_arguments))

        options = dict()
        if cli_arguments.config_file:
            if not os.path.isfile(cli_arguments.config_file):
                print "The provided configuration file does not exist."
                exit(1)

            config_file_options.read(cli_arguments.config_file)

        options["host"] = cli_arguments.host or config_file_options.get("Server", "host")
        options["port"] = cli_arguments.port or config_file_options.get("Server", "port")
        options["username"] = cli_arguments.username or config_file_options.get("Server", "username")
        options["password"] = cli_arguments.password or config_file_options.get("Server", "password")
        options["vhost"] = cli_arguments.vhost or config_file_options.get("Server", "vhost")
        options["check_rate"] = cli_arguments.check_rate or config_file_options.getfloat("Server", "check_rate")
        options["queues"] = cli_arguments.queues or config_file_options.get("Server", "queues")
        options["queues"] = options["queues"].split(",")

        options["email_to"] = cli_arguments.email_to or config_file_options.get("Email", "to")
        options["email_to"] = options["email_to"].split(",")
        options["email_from"] = cli_arguments.email_from or config_file_options.get("Email", "from")
        options["email_subject"] = cli_arguments.email_subject or config_file_options.get("Email", "subject")
        options["email_server"] = cli_arguments.email_server or config_file_options.get("Email", "host")
        options["email_password"] = cli_arguments.email_password or config_file_options.get("Email", "password")
        options["slack_url"] = cli_arguments.slack_url or config_file_options.get("Slack", "url")
        options["slack_channel"] = cli_arguments.slack_channel or config_file_options.get("Slack", "channel")
        options["slack_username"] = cli_arguments.slack_username or config_file_options.get("Slack", "username")
        options["telegram_bot_id"] = cli_arguments.telegram_bot_id or config_file_options.get("Telegram", "bot_id")
        options["telegram_channel"] = cli_arguments.telegram_channel or config_file_options.get("Telegram", "channel")

        if config_file_options.has_section("Email"):
            options["email_ssl"] = config_file_options.getboolean("Email", "ssl")
        else:
            options["email_ssl"] = cli_arguments.email_ssl

        # get queue specific condition values if any, else construct from the generic one
        conditions = OptionsResover.construct_conditions(options, cli_arguments, config_file_options)
        options = dict(options.items() + conditions.items())

        return options

    @staticmethod
    def construct_conditions(options, cli_arguments, config_file_options):
        conditions = dict()

        # get the generic condition values from the "[Conditions]" section
        default_conditions = dict()
        try:
            default_conditions["ready_queue_size"] = cli_arguments.ready_queue_size or config_file_options.getint("Conditions", "ready_queue_size")
            default_conditions["unack_queue_size"] = cli_arguments.unack_queue_size or config_file_options.getint("Conditions", "unack_queue_size")
            default_conditions["total_queue_size"] = cli_arguments.total_queue_size or config_file_options.getint("Conditions", "total_queue_size")
            default_conditions["consumers_connected"] = cli_arguments.consumers_connected or config_file_options.getint("Conditions", "consumers_connected")
            default_conditions["open_connections"] = cli_arguments.open_connections or config_file_options.getint("Conditions", "open_connections")
            default_conditions["nodes_running"] = cli_arguments.nodes_running or config_file_options.getint("Conditions", "nodes_running")
            default_conditions["node_memory_used"] = cli_arguments.node_memory_used or config_file_options.getint("Conditions", "node_memory_used")
        except:
            pass

        # check if queue specific condition sections exist, if not use the generic conditions
        if "queues" in options:
            for queue in options["queues"]:
                queue_conditions_section_name = "Conditions:" + queue

                try:
                    queue_conditions = dict()
                    queue_conditions["ready_queue_size"] = cli_arguments.ready_queue_size or config_file_options.getint(queue_conditions_section_name, "ready_queue_size")
                    queue_conditions["unack_queue_size"] = cli_arguments.unack_queue_size or config_file_options.getint(queue_conditions_section_name, "unack_queue_size")
                    queue_conditions["total_queue_size"] = cli_arguments.total_queue_size or config_file_options.getint(queue_conditions_section_name, "total_queue_size")
                    queue_conditions["consumers_connected"] = cli_arguments.consumers_connected or config_file_options.getint(queue_conditions_section_name, "consumers_connected")
                    queue_conditions["open_connections"] = cli_arguments.open_connections or config_file_options.getint(queue_conditions_section_name, "open_connections")
                    queue_conditions["nodes_running"] = cli_arguments.nodes_running or config_file_options.getint(queue_conditions_section_name, "nodes_running")
                    queue_conditions["node_memory_used"] = cli_arguments.node_memory_used or config_file_options.getint(queue_conditions_section_name, "node_memory_used")
                    conditions[queue] = queue_conditions
                except:
                    conditions[queue] = default_conditions

        return {"conditions": conditions}



#! /usr/bin/python2

from setuptools import setup, find_packages

setup(
    name="rabbitmq-alert",
    packages=find_packages(exclude=["test*"]),
    version="1.0.2",
    description="Send notifications when predefined conditions are met",
    author="Germano Fronza, Kostas Milonas",
    author_email="germano.inf@gmail.com",
    url="https://github.com/gfronza/rabbitmq-alert",
    download_url="https://github.com/gfronza/rabbitmq-alert/tarball/1.0.2",
    keywords=["rabbitmq", "alert", "monitor"],
    classifiers=[]
)
